// $Id$
//
//    File: FCALTreeProc.cc
// Created: Wed Aug 10 13:17:20 EST 2005
// Creator: shepherd (on Darwin 129-79-159-16.dhcp-bl.indiana.edu 7.9.0 powerpc)
//
#include "FCALTreeProc.h"
#include "JANA/JApplication.h"
#include "DANA/DApplication.h"

#include "PID/DKinematicData.h"
#include "TRACKING/DMCThrown.h"
#include "TRACKING/DMagneticFieldStepper.h"
#include "HDGEOMETRY/DMagneticFieldMap.h"
#include "FCAL/DFCALGeometry.h"
#include "FCAL/DFCALTruthShower.h"
#include "FCAL/DFCALHit.h"
#include "FCAL/DFCALCluster.h"
#include "FCAL/DFCALPhoton.h"

#include "BCAL/DBCALTruthShower.h"
#include "BCAL/DBCALShower.h"
#include "BCAL/DBCALPhoton.h"


#include "PID/DPhoton.h"
//#include "PID/DPi0.h"
#include "PID/DTwoGammaFit.h"
#include "PID/DParticle.h"

#include "PID/DChargedTrack.h"
#include "PID/DKinFit.h"
#include "PID/DTwoGammaFit_factory.h"

#include "TBranch.h"
#include "TH1F.h"
#include "TH2F.h"

#include "HDDM/DEventSourceHDDM.h"
#include "HDDM/hddm_s.h"

#include <DLorentzVector.h>
#include <TLorentzRotation.h>

#ifndef PI
 #define PI 3.1415927
#endif

#define MC_THROWN
//#define FCAL_TRUTH 1
//#define FCAL_HITS 1
//#define FCAL_CLUSTERS 1
//#define FCAL_PHOTONS 1

//#define BCAL_SHOWERS 1
//#define BCAL_TRUTH 1
//#define BCAL_PHOTONS 1

//#define PID_PHOTONS 1
//#define PID_FPI0 1
//#define PID_FETA 1
//#define PID_PARTICLES 1
#define PID_ETAPI 1


// The first approximation for shower z-position is FCAL mid-plane
/* static float FCAL_Zlen = 45.0;
static float FCAL_Zmin = 546.5;
static float FCAL_Zmid = FCAL_Zmin+FCAL_Zlen/2.0; */


//------------------
// init
//------------------
jerror_t FCALTreeProc::init(void)
{
	filename = "reconstructed.hddm";
	file = init_s_HDDM((char*)filename.c_str());
	Nevents_written = 0;
      
  //array = new TBranchClones("TVector3");

 _bcal_phot_calPos = new TVector3[PID_PHOT_MAX];
 _fcal_phot_calPos = new TVector3[PID_PHOT_MAX];
 _partMom = new  TVector3[PID_PART_MAX];
 _bcal_swum_partPos = new TVector3[PID_PART_MAX];
 _fcal_swum_partPos = new TVector3[PID_PART_MAX];
 _bcal_swum_partMom = new TVector3[PID_PART_MAX];
 _fcal_swum_partMom = new TVector3[PID_PART_MAX];

	// Create histograms here
	m_treeFile = new TFile( "fcaltree4.root", "RECREATE" );
        m_tree = new TTree("Fcal","FCAL Tree");
	//     m_outFile = new TFile( outFile.c_str(), "recreate" );
        m_outTree = new TTree( "kin", "Kinematics" );




#ifdef MC_THROWN


	m_tree->Branch( "TnPhotons",&TnPhotons,"TnPhotons/I");
	m_tree->Branch( "TnProtons",&TnProtons,"TnProtons/I");
        m_tree->Branch( "nThrown",&nThrown,"nThrown/I");
        m_tree->Branch( "nCharged",&nCharged,"nCharged/I");

        m_tree->Branch( "ThrownType",thrownType,"ThrownType[nThrown]/I");
        m_tree->Branch( "ThrownPDG",thrownTypePDG,"ThrownPDG[nThrown]/I");
        m_tree->Branch( "ThrownMyID",thrownMID,"ThrownMyID[nThrown]/I");
        m_tree->Branch( "ThrownParent",thrownParent,"ThrownParent[nThrown]/I");
        m_tree->Branch( "ThrownQ",thrownQ,"ThrownQ[nThrown]/F");
        m_tree->Branch( "ThrownEnergy",thrownEnergy,"ThrownEnergy[nThrown]/F");
        m_tree->Branch( "ThrownEnergyTot",&thrownEnergyTot,"ThrownEnergyTot/F");
	m_tree->Branch( "ThrownNeutEnergyTot",&thrownNeutEnergyTot,"ThrownNeutEnergyTot/F");
        m_tree->Branch( "ThrownP",thrownP,"ThrownP[nThrown]/F");
        m_tree->Branch( "ThrownMass",thrownMass,"ThrownMass[nThrown]/F");
        m_tree->Branch( "ThrownPhi",thrownPhi,"ThrownPhi[nThrown]/F");
        m_tree->Branch( "ThrownTheta",thrownTheta,"ThrownTheta[nThrown]/F");
        m_tree->Branch( "ThrownVx",thrownVx,"ThrownVx[nThrown]/F");
        m_tree->Branch( "ThrownVy",thrownVy,"ThrownVy[nThrown]/F");
        m_tree->Branch( "ThrownVz",thrownVz,"ThrownVz[nThrown]/F");

        m_tree->Branch( "TnPairs",&TnPairs,"TnPairs/I");


        m_tree->Branch( "Tpi0_mass",Tpi0_mass,"Tpi0_mass[TnPairs]/F");
        m_tree->Branch( "Tpi0_prob",Tpi0_prob,"Tpi0_prob[TnPairs]/F");
        m_tree->Branch( "Tpi0_Chi2",Tpi0_Chi2,"Tpi0_Chi2[TnPairs]/F");
        m_tree->Branch( "Tpi0_pull_1",Tpi0_pull_1,"Tpi0_pull_1[TnPairs]/F");
        m_tree->Branch( "Tpi0_pull_2",Tpi0_pull_2,"Tpi0_pull_2[TnPairs]/F");
        m_tree->Branch( "Tpi0_pull_3",Tpi0_pull_3,"Tpi0_pull_3[TnPairs]/F");
        m_tree->Branch( "Tpi0_pull_4",Tpi0_pull_4,"Tpi0_pull_4[TnPairs]/F");
        m_tree->Branch( "Tpi0_pull_5",Tpi0_pull_5,"Tpi0_pull_5[TnPairs]/F");
        m_tree->Branch( "Tpi0_pull_6",Tpi0_pull_6,"Tpi0_pull_6[TnPairs]/F");
        m_tree->Branch( "Tpi0_momentum_Mag",Tpi0_momentum_Mag,"Tpi0_momentum_Mag[TnPairs]/F");
        m_tree->Branch( "Tpi0_energy",Tpi0_energy,"Tpi0_energy[TnPairs]/F");


        m_tree->Branch( "Teta_mass",Teta_mass,"Teta_mass[TnPairs]/F");
        m_tree->Branch( "Teta_prob",Teta_prob,"Teta_prob[TnPairs]/F");
        m_tree->Branch( "Teta_Chi2",Teta_Chi2,"Teta_Chi2[TnPairs]/F");
        m_tree->Branch( "Teta_pull_1",Teta_pull_1,"Teta_pull_1[TnPairs]/F");
        m_tree->Branch( "Teta_pull_2",Teta_pull_2,"Teta_pull_2[TnPairs]/F");
        m_tree->Branch( "Teta_pull_3",Teta_pull_3,"Teta_pull_3[TnPairs]/F");
        m_tree->Branch( "Teta_pull_4",Teta_pull_4,"Teta_pull_4[TnPairs]/F");
        m_tree->Branch( "Teta_pull_5",Teta_pull_5,"Teta_pull_5[TnPairs]/F");
        m_tree->Branch( "Teta_pull_6",Teta_pull_6,"Teta_pull_6[TnPairs]/F");
        m_tree->Branch( "Teta_momentum_Mag",Teta_momentum_Mag,"Teta_momentum_Mag[TnPairs]/F");
        m_tree->Branch( "Teta_energy",Teta_energy,"Teta_energy[TnPairs]/F");

        m_tree->Branch( "TnCandChi2_3",&TnCandChi2_3,"TnCandChi2_3/I");
        m_tree->Branch( "TnCandChi2_5",&TnCandChi2_5,"TnCandChi2_5/I");
        m_tree->Branch( "TnFitCandChi2_10",&TnFitCandChi2_10,"TnFitCandChi2_10/I");



        m_tree->Branch( "TProbMax",&TProbMax,"TProbMax/F");
        m_tree->Branch( "TProbMax_PI0",&TProbMax_PI0,"TProbMax_PI0/F");
        m_tree->Branch( "TProbMax_ETA",&TProbMax_ETA,"TProbMax_ETA/F");
        m_tree->Branch( "TProbMax_FIT",&TProbMax_FIT,"TProbMax_FIT/F");

        m_tree->Branch( "TPI0CHI2",&TPI0CHI2,"TPI0CHI2/F");
        m_tree->Branch( "TETACHI2",&TETACHI2,"TETACHI2/F");
        m_tree->Branch( "TFITCHI2",&TFITCHI2,"TFITCHI2/F");
        m_tree->Branch( "TFITNDF",&TFITNDF,"TFITNDF/F");



        m_tree->Branch( "TETAMASS",&TETAMASS,"TETAMASS/F");
        m_tree->Branch( "TPI0MASS",&TPI0MASS,"TPI0MASS/F");

        m_tree->Branch( "TRESMASS",&TRESMASS,"TRESMASS/F");
        m_tree->Branch( "TRESENERGY",&TRESENERGY,"TRESENERGY/F");
        m_tree->Branch( "TRESMOM",&TRESMOM,"TRESMOM/F");
        m_tree->Branch( "TRESPT",&TRESPT,"TRESPT/F");


        m_tree->Branch( "TMISMASS",&TMISMASS,"TMISMASS/F");
        m_tree->Branch( "TMISENERGY",&TMISENERGY,"TMISENERGY/F");
        m_tree->Branch( "TMISMOM",&TMISMOM,"TMISMOM/F");
        m_tree->Branch( "TMISPT",&TMISPT,"TMISPT/F");

        m_tree->Branch( "TTOTALPT",&TTOTALPT,"TTOTALPT/F");


        m_tree->Branch( "TRECMASS",&TRECMASS,"TRECMASS/F");
        m_tree->Branch( "TRECMOM",&TRECMOM,"TRECMOM/F");
        m_tree->Branch( "TRECENERGY",&TRECENERGY,"TRECENERGY/F");
        m_tree->Branch( "TRECPT",&TRECPT,"TRECPT/F");

	m_tree->Branch( "thrownlowresphot_energy",&thrownlowresphot_energy,"thrownlowresphot_energy/F");
        m_tree->Branch( "thrownlowresphot_theta",&thrownlowresphot_theta,"thrownlowresphot_theta/F");
		



  m_tree->Branch( "Tt", &Tt, "Tt/F" );
  m_tree->Branch( "TcosGJ", &TcosGJ, "TcosGJ/F" );
  m_tree->Branch( "TphiGJ", &TphiGJ, "TphiGJ/F" );

#endif




#ifdef PID_ETAPI
        m_tree->Branch( "nPairs",&nPairs,"nPairs/I");
        m_tree->Branch( "nPhotons",&nPhotons,"nPhotons/I");
        m_tree->Branch( "nPhotons_ps",&nPhotons_ps,"nPhotons_ps/I");

        m_tree->Branch( "pi0_mass",pi0_mass,"pi0_mass[nPairs]/F");
        m_tree->Branch( "pi0_prob",pi0_prob,"pi0_prob[nPairs]/F");
        m_tree->Branch( "pi0_Chi2",pi0_Chi2,"pi0_Chi2[nPairs]/F");
        m_tree->Branch( "pi0_pull_1",pi0_pull_1,"pi0_pull_1[nPairs]/F");
        m_tree->Branch( "pi0_pull_2",pi0_pull_2,"pi0_pull_2[nPairs]/F");
        m_tree->Branch( "pi0_pull_3",pi0_pull_3,"pi0_pull_3[nPairs]/F");
        m_tree->Branch( "pi0_pull_4",pi0_pull_4,"pi0_pull_4[nPairs]/F");
        m_tree->Branch( "pi0_pull_5",pi0_pull_5,"pi0_pull_5[nPairs]/F");
        m_tree->Branch( "pi0_pull_6",pi0_pull_6,"pi0_pull_6[nPairs]/F");
        m_tree->Branch( "pi0_momentum_Mag",pi0_momentum_Mag,"pi0_momentum_Mag[nPairs]/F");
        m_tree->Branch( "pi0_energy",pi0_energy,"pi0_energy[nPairs]/F");


        m_tree->Branch( "eta_mass",eta_mass,"eta_mass[nPairs]/F");
        m_tree->Branch( "eta_prob",eta_prob,"eta_prob[nPairs]/F");
        m_tree->Branch( "eta_Chi2",eta_Chi2,"eta_Chi2[nPairs]/F");
        m_tree->Branch( "eta_pull_1",eta_pull_1,"eta_pull_1[nPairs]/F");
        m_tree->Branch( "eta_pull_2",eta_pull_2,"eta_pull_2[nPairs]/F");
        m_tree->Branch( "eta_pull_3",eta_pull_3,"eta_pull_3[nPairs]/F");
        m_tree->Branch( "eta_pull_4",eta_pull_4,"eta_pull_4[nPairs]/F");
        m_tree->Branch( "eta_pull_5",eta_pull_5,"eta_pull_5[nPairs]/F");
        m_tree->Branch( "eta_pull_6",eta_pull_6,"eta_pull_6[nPairs]/F");
        m_tree->Branch( "eta_momentum_Mag",eta_momentum_Mag,"eta_momentum_Mag[nPairs]/F");
        m_tree->Branch( "eta_energy",eta_energy,"eta_energy[nPairs]/F");

        m_tree->Branch( "gamma4_energy",gamma4_energy,"gamma4_energy[nPairs]/F");
        m_tree->Branch( "gamma4_pt",gamma4_pt,"gamma4_pt[nPairs]/F");
        m_tree->Branch( "nCandChi2_3",&nCandChi2_3,"nCandChi2_3/I");
        m_tree->Branch( "nCandChi2_5",&nCandChi2_5,"nCandChi2_5/I");
        m_tree->Branch( "nFitCandChi2_50",&nFitCandChi2_50,"nFitCandChi2_50/I"); 
        m_tree->Branch( "nFitCandProb",&nFitCandProb,"nFitCandProb/I");

	m_tree->Branch( "neutEtot",&neutEtot,"neutEtot/F");


        m_tree->Branch( "ProbMax",&ProbMax,"ProbMax/F");
        m_tree->Branch( "ProbMax_PI0",&ProbMax_PI0,"ProbMax_PI0/F");
        m_tree->Branch( "ProbMax_ETA",&ProbMax_ETA,"ProbMax_ETA/F");
        m_tree->Branch( "ProbMax_FIT",&ProbMax_FIT,"ProbMax_FIT/F");

        m_tree->Branch( "PI0CHI2",&PI0CHI2,"PI0CHI2/F");
        m_tree->Branch( "ETACHI2",&ETACHI2,"ETACHI2/F");
        m_tree->Branch( "FITCHI2",&FITCHI2,"FITCHI2/F");
        m_tree->Branch( "FITNDF",&FITNDF,"FITNDF/F");

	m_tree->Branch( "PULL",PULL,"PULL[25]/D");
	//	m_tree->Branch( "PULL0",&PULL0,"PULL0/F");
	//	m_tree->Branch( "PULL1",&PULL1,"PULL1/F");
	//	m_tree->Branch( "PULL2",&PULL2,"PULL2/F");
	//	m_tree->Branch( "PULL3",&PULL3,"PULL3/F");
	//	m_tree->Branch( "PULL4",&PULL4,"PULL4/F");

        m_tree->Branch( "ETAMASS",&ETAMASS,"ETAMASS/F");
        m_tree->Branch( "PI0MASS",&PI0MASS,"PI0MASS/F");

        m_tree->Branch( "RESMASS",&RESMASS,"RESMASS/F");
        m_tree->Branch( "RESENERGY",&RESENERGY,"RESENERGY/F");
        m_tree->Branch( "RESMOM",&RESMOM,"RESMOM/F");
        m_tree->Branch( "RESPT",&RESPT,"RESPT/F");


        m_tree->Branch( "MISMASS",&MISMASS,"MISMASS/F");
        m_tree->Branch( "MISMASS2",&MISMASS2,"MISMASS2/F");

        m_tree->Branch( "MISENERGY",&MISENERGY,"MISENERGY/F");
        m_tree->Branch( "MISMOM",&MISMOM,"MISMOM/F");
        m_tree->Branch( "MISPT",&MISPT,"MISPT/F");

        m_tree->Branch( "TOTALPT",&TOTALPT,"TOTALPT/F");


        m_tree->Branch( "RECMASS",&RECMASS,"RECMASS/F");
        m_tree->Branch( "RECMOM",&RECMOM,"RECMOM/F");
        m_tree->Branch( "RECENERGY",&RECENERGY,"RECENERGY/F");
        m_tree->Branch( "RECPT",&RECPT,"RECPT/F");

        m_tree->Branch( "resphot_energy",resphot_energy,"resphot_energy[4]/F");
        m_tree->Branch( "resphot_theta",resphot_theta,"resphot_theta[4]/F");
	m_tree->Branch( "lowresphot_energy",&lowresphot_energy,"lowresphot_energy/F");
        m_tree->Branch( "lowresphot_theta",&lowresphot_theta,"lowresphot_theta/F");

;

        m_tree->Branch( "resphot_in_energy",resphot_in_energy,"resphot_in_energy[4]/F");
        m_tree->Branch( "resphot_in_theta",resphot_in_theta,"resphot_in_theta[4]/F");
	m_tree->Branch( "lowresphot_in_energy",&lowresphot_in_energy,"lowresphot_in_energy/F");
        m_tree->Branch( "lowresphot_in_theta",&lowresphot_in_theta,"lowresphot_in_theta/F");


	m_tree->Branch( "etapair_energydiff",&etapair_energydiff,"etapair_energydiff/F");
	m_tree->Branch( "pi0pair_energydiff",&pi0pair_energydiff,"pi0pair_energydiff/F");
		



  m_tree->Branch( "t", &t, "t/F" );
  m_tree->Branch( "cosGJ", &cosGJ, "cosGJ/F" );
  m_tree->Branch( "phiGJ", &phiGJ, "phiGJ/F" );

   m_tree->Branch( "pi0theta_g1", &pi0theta_g1, " pi0theta_g1/F" );
   m_tree->Branch( "pi0theta_g2", &pi0theta_g2, " pi0theta_g2/F" );
   m_tree->Branch( "pi0phi_g1", &pi0phi_g1, " pi0phi_g1/F" );
   m_tree->Branch( "pi0phi_g2", &pi0phi_g2, " pi0phi_g2/F" );
   m_tree->Branch( "pi0e_g1", &pi0e_g1, " pi0e_g1/F" );
   m_tree->Branch( "pi0e_g2", &pi0e_g2, " pi0e_g2/F" );

   m_tree->Branch( "pi0theta_g1_com", pi0theta_g1_com, " pi0theta_g1_com[nPairs]/F" );
   m_tree->Branch( "pi0theta_g2_com", pi0theta_g2_com, " pi0theta_g2_com[nPairs]/F" );
   m_tree->Branch( "pi0phi_g1_com", pi0phi_g1_com, " pi0phi_g1_com[nPairs]/F" );
   m_tree->Branch( "pi0phi_g2_com", pi0phi_g2_com, " pi0phi_g2_com[nPairs]/F" );
   m_tree->Branch( "pi0e_g1_com", pi0e_g1_com, " pi0e_g1_com[nPairs]/F" );
   m_tree->Branch( "pi0e_g2_com", pi0e_g2_com, " pi0e_g2_com[nPairs]/F" );


   m_tree->Branch( "etatheta_g1", &etatheta_g1, " etatheta_g1/F" );
   m_tree->Branch( "etatheta_g2", &etatheta_g2, " etatheta_g2/F" );
   m_tree->Branch( "etaphi_g1", &etaphi_g1, " etaphi_g1/F" );
   m_tree->Branch( "etaphi_g2", &etaphi_g2, " etaphi_g2/F" );
   m_tree->Branch( "etae_g1", &etae_g1, " etae_g1/F" );
   m_tree->Branch( "etae_g2", &etae_g2, " etae_g2/F" );

   m_tree->Branch( "etatheta_g1_com", etatheta_g1_com, " etatheta_g1_com[nPairs]/F" );
   m_tree->Branch( "etatheta_g2_com", etatheta_g2_com, " etatheta_g2_com[nPairs]/F" );
   m_tree->Branch( "etaphi_g1_com", etaphi_g1_com, " etaphi_g1_com[nPairs]/F" );
   m_tree->Branch( "etaphi_g2_com", etaphi_g2_com, " etaphi_g2_com[nPairs]/F" );
   m_tree->Branch( "etae_g1_com", etae_g1_com, " etae_g1_com[nPairs]/F" );
   m_tree->Branch( "etae_g2_com", etae_g2_com, " etae_g2_com[nPairs]/F" );


        m_outTree->Branch( "nPart", &nPart, "nPart/I" );
	
	m_outTree->Branch( "e", m_e, "e[nPart]/F" );
        m_outTree->Branch( "px", m_px, "px[nPart]/F" );
        m_outTree->Branch( "py", m_py, "py[nPart]/F" );
	m_outTree->Branch( "pz", m_pz, "pz[nPart]/F" );
        m_outTree->Branch( "eBeam", &m_eBeam, "eBeam/F" );
        m_outTree->Branch( "pxBeam", &m_pxBeam, "pxBeam/F" );
        m_outTree->Branch( "pyBeam", &m_pyBeam, "pyBeam/F" );
        m_outTree->Branch( "pzBeam", &m_pzBeam, "pzBeam/F" );
        m_outTree->Branch( "eRecoil", &m_eRecoil, "eRecoil/F" );
        m_outTree->Branch( "pxRecoil", &m_pxRecoil, "pxRecoil/F" );
        m_outTree->Branch( "pyRecoil", &m_pyRecoil, "pyRecoil/F" );
        m_outTree->Branch( "pzRecoil", &m_pzRecoil, "pzRecoil/F" );

  
	//	m_tree->Branch( "FITP4_RES","TLorentzVector",&FITP4_RES, 128000,0); //TLorentzVector of resonance




#endif

       return NOERROR;
}

//------------------
// brun
//------------------
jerror_t FCALTreeProc::brun(JEventLoop *eventLoop, int runnumber)
{
	return NOERROR;
}

//------------------
// evnt
//------------------
jerror_t FCALTreeProc::evnt(JEventLoop *loop, int eventnumber)
{
	// Fill histograms here
        int nHits = 0;
        float Etot=0;
	write_out = false;   

  // This is a little complicated. We need to get a hold of the s_HDDM_t
  // structure pointer for this event so we can pass it to flush_s_HDDM()
  // along with our ouput stream pointer. The flush routine frees up the
  // memory in the s_HDDM_t structure. When the framework tries "flush"ing
  // a second time, we get a seg fault. To prevent the framework from
  // flushing, we have to clear the free_on_flush flag (by default set
  // to true). This means we need to get the DEventSource pointer and
  // downcast to a DEventSourceHDDM structure. It a little strange setting
  // this for every event, but we have no way of knowing when the event
  // source changes and this at least guarantees it for all event sources.
  JEvent& event = loop->GetJEvent();
  JEventSource *source = event.GetJEventSource();
  DEventSourceHDDM *hddm_source = dynamic_cast<DEventSourceHDDM*>(source);
  if(!hddm_source){
    cerr<<" This program MUST be used with an HDDM file as input!"<<endl;
    exit(-1);
  }
  s_HDDM_t *hddm = (s_HDDM_t*)event.GetRef();
  if(!hddm)return NOERROR;


	////////////////////////////////////////////
	//////begin reconstructing Truth information
	////////////////////////////////////////////



#ifdef MC_THROWN
	TnProtons=0;
	TnPhotons=0;
	thrownNeutEnergyTot = 0.0;
	vector<const DMCThrown*> thrownVect;
	loop->Get( thrownVect );
	TLorentzVector thrownResLV(0,0,0,0);
        thrownlowresphot_energy = 20.0;
	thrownlowresphot_theta = 20.0;
	vector<const DMCThrown*> thrownVect_proton;
	nCharged = 0;
 	   nThrown = thrownVect.size();
        assert( nThrown < MC_THROWN_MAX);
        nHits=0;
	for( vector<const DMCThrown*>::const_iterator thrown = thrownVect.begin();
		thrown != thrownVect.end(); ++thrown ) {

           thrownType[nHits] = (**thrown).type;
	   if ( thrownType[nHits]==1) TnPhotons++;
	   if ( thrownType[nHits]==14) {
	     TnProtons++;
	     thrownVect_proton.push_back(*(thrown));}

           thrownTypePDG[nHits] = (**thrown).pdgtype;
           thrownMID[nHits] = (**thrown).myid;
           thrownParent[nHits] = (**thrown).parentid;
           thrownEnergy[nHits] = (**thrown).energy();
	   //   cout << thrownEnergy[nHits] << " " << (**thrown).energy() << endl;
           thrownP[nHits] = (**thrown).pmag();
           thrownMass[nHits] = (**thrown).mass();
           thrownPhi[nHits] = (**thrown).momentum().Phi();
           thrownTheta[nHits] = (**thrown).momentum().Theta();
           thrownQ[nHits] = (**thrown).charge();
	   if((**thrown).charge() != 0) nCharged++;

           thrownVx[nHits] = (**thrown).x();
           thrownVy[nHits] = (**thrown).y();
           thrownVz[nHits] = (**thrown).z();
	   TLorentzVector ThrownLV = (**thrown).lorentzMomentum();
	   Etot += (**thrown).energy();
	   if((**thrown).charge() == 0 && (**thrown).type != 0)   thrownNeutEnergyTot+= (**thrown).energy();
	   if (thrownType[nHits] == 1){

	     thrownResLV += ThrownLV;

	     if (thrownlowresphot_energy > thrownEnergy[nHits]) {
		      thrownlowresphot_energy = thrownEnergy[nHits];
		      thrownlowresphot_theta = thrownTheta[nHits];
		    }
	   }
	   


           /* This structure worked for particle gun with just one thrown particle
           m_thrown.p = (**thrown).p;
           m_thrown.E = (**thrown).E;
           m_thrown.the = (**thrown).theta;
           m_thrown.phi = (**thrown).phi;
           m_thrown.M = (**thrown).mass;
           m_thrown.vertex[0] = (**thrown).x;
           m_thrown.vertex[1] = (**thrown).y;
           m_thrown.vertex[2] = (**thrown).z;*/
           nHits++;
        }

	thrownResMass = thrownResLV.M();
	thrownEnergyTot = Etot;

  



       

       TProbMax = 0.0;
       TProbMax_PI0 = 0.0;
       TProbMax_ETA = 0.0;
       TETACHI2 = 0.0;
       TPI0CHI2 = 0.0;
    
       TnPairs = 0;
       TPI0MASS = 0.0;
       TETAMASS = 0.0;
 
       TRECMASS = 0.0;
       TRECENERGY= 0.0;
       TRECMOM = 0.0;
       TRECPT = 0.0;


       TRESMASS = 0.0;
       TRESENERGY= 0.0;
       TMISMASS = 0.0;
       TMISENERGY = 0.0;
       
       TRESPT = 0.0;
       TTOTALPT = 0.0;
       float TProb = 0.0;
       TProbMax = 0.0;
       TcosGJ=0.0;
       TphiGJ=0.0;
       Tt=0.0;
       TnPart=2;
       TnCandChi2_3 = 0;
       TnCandChi2_5 = 0;
       TnFitCandChi2_10 = 0;


       TLorentzVector TP4_beam( 0.0, 0.0, 9.0, 9.0 );       
       TLorentzVector TP4_target( 0, 0, 0, 0.938 );
       TLorentzVector TP4_RECOIL(0,0,0,0);

       if ( TnPhotons==4 && TnProtons==1 ){

       TP4_RECOIL = thrownVect_proton[0]->lorentzMomentum();




       //   DKinematicData Tkd_beam = DKinematicData();
       //	DKinematicData Tkd_target = DKinematicData();
       //	vector<DKinematicData> Tkd_initialState;
       //	vector<DKinematicData> Tkd_finalState;
       //	vector<int> TconstraintParticles;
       //	vector<DKinematicData> Tkindata_PI0;
       //	vector<DKinematicData> Tkinout_PI0 ;
       //	DVector3 Tvertex_PI0;
       //	TLorentzVector TP4_PI0;
       //	vector<DKinematicData> Tkindata_ETA;
       //	vector<DKinematicData> Tkinout_ETA;
       //	DVector3 Tvertex_ETA;
       //	TLorentzVector TP4_ETA;
       //	TLorentzVector TP4_MISS4M;

        for(unsigned int i = 0; i < thrownVect.size() ; i++) {
            if ( thrownVect[i]->type != 1  ) continue;

            for (unsigned int j = i+1; j < thrownVect.size() ; j++) {
                if ( thrownVect[j]->type != 1) continue;

		 for (unsigned int k = 0; k < thrownVect.size() ; k++) {
		   if ( thrownVect[k]->type != 1 ||  k == i || k == j ) continue;

		   for (unsigned int l = k+1; l < thrownVect.size() ; l++) {
		     if ( thrownVect[l]->type != 1 || l == i || l == j )continue;



		////////////////////////////////
		///Fit Initial and Final States
		////////////////////////////////

		
		       vector<DKinematicData> Tkd_initialState;


		        DKinematicData Tkd_beam = DKinematicData();
		Tkd_beam.setMass(0.0);
		Tkd_beam.setCharge(0);
		Tkd_beam.setMassFixed();
		Tkd_beam.setMomentum( TVector3(0.0, 0.0, 9.0));
		Tkd_beam.clearErrorMatrix();
		Tkd_initialState.push_back(Tkd_beam);

		   DKinematicData Tkd_target = DKinematicData();
		Tkd_target.setMass(0.93827);
		Tkd_target.setCharge(1);
		Tkd_target.setMassFixed();
		Tkd_target.setMomentum( TVector3(0.0, 0.0, 0.0));
		Tkd_target.clearErrorMatrix();
		Tkd_initialState.push_back(Tkd_target);

			 vector<DKinematicData> Tkd_finalState;
	
		 Tkd_finalState.push_back(*(thrownVect[i]));
		 Tkd_finalState.push_back(*(thrownVect[j]));
		 Tkd_finalState.push_back(*(thrownVect[k]));
		 Tkd_finalState.push_back(*(thrownVect[l]));
		 Tkd_finalState.push_back(*(thrownVect_proton[0]));
		 Tkd_finalState[0].clearErrorMatrix();
		 Tkd_finalState[1].clearErrorMatrix();		 
		 Tkd_finalState[2].clearErrorMatrix();
		 Tkd_finalState[3].clearErrorMatrix();
		 Tkd_finalState[4].clearErrorMatrix();

		 DKinFit *Tkfit = new DKinFit(); 
		 Tkfit->SetVerbose(0);	    
		 Tkfit->SetInitial(Tkd_initialState);
		 Tkfit->SetFinal(Tkd_finalState);

		 	 vector<int> TconstraintParticles;
		 TconstraintParticles.push_back(0);
		 TconstraintParticles.push_back(1);
		 Tkfit->SetMassConstraint(0.13498,TconstraintParticles);

		 TconstraintParticles.clear();
		 TconstraintParticles.push_back(2);
		 TconstraintParticles.push_back(3);
		 Tkfit->SetMassConstraint(0.54775,TconstraintParticles);

		 Tkfit->Fit();
		 
		 TFitProb[TnPairs] = Tkfit->Prob();
		 TFitChi2[TnPairs] = Tkfit->Chi2();
		 TFitNDF[TnPairs] = Tkfit->Ndf();

		 TFitPull0[TnPairs] = Tkfit->GetPull(0);
		 TFitPull1[TnPairs] = Tkfit->GetPull(1);
		 TFitPull2[TnPairs] = Tkfit->GetPull(2);
		 TFitPull3[TnPairs] = Tkfit->GetPull(3);
		 TFitPull4[TnPairs] = Tkfit->GetPull(4);

		 delete Tkfit;

		 /////////////////////////
		 //Fitting Two Gamma to Pi0
		 /////////////////////////
		     
                DKinFit *Tkfit_PI0 = new DKinFit(); 
                Tkfit_PI0->SetVerbose(0);

			 vector<DKinematicData> Tkindata_PI0;
                Tkindata_PI0.clear();
                Tkindata_PI0.push_back( *thrownVect[i] );
                Tkindata_PI0.push_back( *thrownVect[j] );

                Tkfit_PI0->SetFinal(Tkindata_PI0);
                Tkfit_PI0->FitTwoGammas(0.13498,1.); // second parameter is scale for photon error matrix
 
			 vector<DKinematicData> Tkinout_PI0 = Tkfit_PI0->GetFinal_out();

			 //	Tkinout_PI0 = Tkfit_PI0->GetFinal_out();
		Tkinout_PI0 = Tkfit_PI0->GetFinal_out();
                if (Tkinout_PI0.size() != 2) continue;

                DTwoGammaFit* Tfit2g_PI0 = new DTwoGammaFit( TnPairs + 1 );

// set two gamma kinematics

		  TLorentzVector TP4_PI0 = thrownVect[i]->lorentzMomentum() + thrownVect[j]->lorentzMomentum(); 

		  //	 TP4_PI0 = thrownVect[i]->lorentzMomentum() + thrownVect[j]->lorentzMomentum();     
		if ( TP4_PI0.Beta() <0 || TP4_PI0.Beta() >= 1) TP4_PI0.SetXYZT(0.,0.,0.,0.);


			DVector3 Tvertex_PI0 = 0.5*(thrownVect[i]->position() + thrownVect[j]->position());
			//	Tvertex_PI0 = 0.5*(thrownVect[i]->position() + thrownVect[j]->position());
                Tfit2g_PI0->setMass( TP4_PI0.M() );
                Tfit2g_PI0->setMomentum( TP4_PI0.Vect() );
                Tfit2g_PI0->setPosition( Tvertex_PI0 );
                Tfit2g_PI0->setProb( Tkfit_PI0->Prob());
                Tfit2g_PI0->setChi2( Tkfit_PI0->Chi2());
               for (int p=0; p < 6; p++) {
                   Tfit2g_PI0->setPulls( Tkfit_PI0->GetPull(p),  p);
                }

                Tfit2g_PI0->setChildMom(  Tkinout_PI0[0].lorentzMomentum() , 0);
                Tfit2g_PI0->setChildMom(  Tkinout_PI0[1].lorentzMomentum() , 1);

		Tpi0_mass[TnPairs] =  Tfit2g_PI0->mass();
		Tpi0_prob[TnPairs] =  Tfit2g_PI0->getProb();
		Tpi0_Chi2[TnPairs] =  Tfit2g_PI0->getChi2();
		Tpi0_pull_1[TnPairs] = Tfit2g_PI0->getPull(0);
		Tpi0_pull_2[TnPairs] = Tfit2g_PI0->getPull(1);
		Tpi0_pull_3[TnPairs] = Tfit2g_PI0->getPull(2);
 		Tpi0_pull_4[TnPairs] = Tfit2g_PI0->getPull(3);
		Tpi0_pull_5[TnPairs] = Tfit2g_PI0->getPull(4);
		Tpi0_pull_6[TnPairs] = Tfit2g_PI0->getPull(5);
		
		Tpi0_momentum_Mag[TnPairs] = TP4_PI0.Vect().Mag();
		Tpi0_energy[TnPairs] = TP4_PI0.T();

 		delete Tkfit_PI0;
		delete Tfit2g_PI0;

                DKinFit *Tkfit_ETA = new DKinFit(); 
                Tkfit_ETA->SetVerbose(0);

		    vector<DKinematicData> Tkindata_ETA;
                Tkindata_ETA.clear();
                Tkindata_ETA.push_back( *thrownVect[k] );
                Tkindata_ETA.push_back( *thrownVect[l] );

                Tkfit_ETA->SetFinal(Tkindata_ETA);
                Tkfit_ETA->FitTwoGammas(0.54775,1.); // second parameter is scale for photon error matrix
 
		  vector<DKinematicData> Tkinout_ETA = Tkfit_ETA->GetFinal_out();
		  //	Tkinout_ETA = Tkfit_ETA->GetFinal_out();

                if (Tkinout_ETA.size() != 2) continue;

                DTwoGammaFit* Tfit2g_ETA = new DTwoGammaFit( TnPairs + 1 );

// set two gamma kinematics
		 TLorentzVector TP4_ETA = thrownVect[k]->lorentzMomentum() + thrownVect[l]->lorentzMomentum(); 
		 //	TP4_ETA = thrownVect[k]->lorentzMomentum() + thrownVect[l]->lorentzMomentum(); 

		if ( TP4_ETA.Beta() < 0 || TP4_ETA.Beta() >= 1) TP4_ETA.SetXYZT(0.,0.,0.,0.);
		
			DVector3 Tvertex_ETA = 0.5*(thrownVect[k]->position() + thrownVect[l]->position());
			//	Tvertex_ETA = 0.5*(thrownVect[k]->position() + thrownVect[l]->position());
                Tfit2g_ETA->setMass( TP4_ETA.M() );
                Tfit2g_ETA->setMomentum( TP4_ETA.Vect() );
                Tfit2g_ETA->setPosition( Tvertex_ETA );
                Tfit2g_ETA->setProb( Tkfit_ETA->Prob());
               Tfit2g_ETA->setChi2( Tkfit_ETA->Chi2());
               for (int p=0; p < 6; p++) {
                   Tfit2g_ETA->setPulls( Tkfit_ETA->GetPull(p),  p);
                }
 
                Tfit2g_ETA->setChildMom(  Tkinout_ETA[0].lorentzMomentum() , 0);
                Tfit2g_ETA->setChildMom(  Tkinout_ETA[1].lorentzMomentum() , 1);


		Teta_mass[TnPairs] =  Tfit2g_ETA->mass();
		Teta_prob[TnPairs] =  Tfit2g_ETA->getProb();
		Teta_Chi2[TnPairs] =  Tfit2g_ETA->getChi2();
		Teta_pull_1[TnPairs] = Tfit2g_ETA->getPull(0);
		Teta_pull_2[TnPairs] = Tfit2g_ETA->getPull(1);
		Teta_pull_3[TnPairs] = Tfit2g_ETA->getPull(2);
 		Teta_pull_4[TnPairs] = Tfit2g_ETA->getPull(3);
		Teta_pull_5[TnPairs] = Tfit2g_ETA->getPull(4);
		Teta_pull_6[TnPairs] = Tfit2g_ETA->getPull(5);
		
		Teta_momentum_Mag[TnPairs] = TP4_ETA.Vect().Mag();
		Teta_energy[TnPairs] = TP4_ETA.T();
		

 		delete Tkfit_ETA;
		delete Tfit2g_ETA;


		  TLorentzVector TP4_MISS4M = TP4_beam + TP4_target - TP4_ETA - TP4_PI0 ;
		  //  TP4_MISS4M = TP4_beam + TP4_target - TP4_ETA - TP4_PI0 ;	
       TMISMASS = TP4_MISS4M.M();
       TMISENERGY = TP4_MISS4M.T();
       TMISMOM = TP4_RECOIL.Vect().Mag();
       TMISPT = TP4_RECOIL.Pt();
     
       TRECMASS = TP4_RECOIL.M();
       TRECENERGY = TP4_RECOIL.T();
       TRECMOM = TP4_RECOIL.Vect().Mag();
       TRECPT = TP4_RECOIL.Pt();
       

       Tt = -( TP4_RECOIL - TP4_target ).Mag2();

       Ttotal_pt[TnPairs] = sqrt( ( TP4_ETA.X() + TP4_PI0.X() + TP4_RECOIL.X()  ) * ( TP4_ETA.X() + TP4_PI0.X() +TP4_RECOIL.X() ) + ( TP4_ETA.Y() + TP4_PI0.Y() + TP4_RECOIL.Y() ) * ( TP4_ETA.Y() + TP4_PI0.Y() + TP4_RECOIL.Y()));

		TProb = Tpi0_prob[TnPairs] *  Teta_prob[TnPairs];
		if ( Tpi0_Chi2[TnPairs] < 3 && Teta_Chi2[TnPairs] < 3 ) TnCandChi2_3++;
		if ( Tpi0_Chi2[TnPairs] < 5 && Teta_Chi2[TnPairs] < 5 ) TnCandChi2_5++;
		if (TFitChi2[TnPairs] < 10) TnFitCandChi2_10++;

		if (TFitProb[TnPairs] > TProbMax){
		  TProbMax = TFitProb[TnPairs];

		  TProbMax_PI0 = Tpi0_prob[TnPairs];
		  TProbMax_ETA = Teta_prob[TnPairs];
		  TProbMax_FIT = TFitProb[TnPairs];
		  TPI0CHI2 = Tpi0_Chi2[TnPairs];
		  TETACHI2 = Teta_Chi2[TnPairs];
		  TFITCHI2 = TFitChi2[TnPairs];
		  TFITNDF = TFitNDF[TnPairs];

		  TPI0MASS = Tpi0_mass[TnPairs];
		  TETAMASS = Teta_mass[TnPairs];

		  TLorentzVector TP4_RES = TP4_PI0 + TP4_ETA;
		  TRESMASS = TP4_RES.M();
		  TRESENERGY = TP4_RES.T();
		  TRESMOM =  TP4_RES.Vect().Mag();
		  TRESPT = TP4_RES.Pt();

		  
		  TTOTALPT = Ttotal_pt[TnPairs];


   if ( (TP4_RES.Beta() > 0 && TP4_RES.Beta() < 1) && TP4_ETA.T() < 10 && TP4_PI0.T() < 10 ) {
		  
		  TLorentzRotation TresRestBoost( -TP4_RES.BoostVector() );


		  

    DLorentzVector Tbeam_res   = TresRestBoost * TP4_beam;
    DLorentzVector Trecoil_res = TresRestBoost * TP4_RECOIL;
    DLorentzVector Tp1_res = TresRestBoost * TP4_PI0;

    
    DVector3 Tz = Tbeam_res.Vect().Unit();
    DVector3 Ty = Trecoil_res.Vect().Cross(Tz).Unit();
    DVector3 Tx = Ty.Cross(Tz);

   
    DVector3 Tangles( (Tp1_res.Vect()).Dot(Tx),
		     (Tp1_res.Vect()).Dot(Ty),
		     (Tp1_res.Vect()).Dot(Tz) );

    TcosGJ = Tangles.CosTheta();
    TphiGJ = Tangles.Phi();
   }		  



		}
		TnPairs++;

		   }
		 }
	    }
	}

       }


#endif

       ////end of Truth reconstruction









       /////////////////////////////////////////
       //////begin reconstructing HDGEANT events
       /////////////////////////////////////////
#ifdef PID_ETAPI

	vector<const DPhoton*> photons;
       loop->Get(photons);
       vector<const DParticle*> recoilproton;
       loop->Get(recoilproton);
       //photon multiplicity
       nPhotons = photons.size();
       neutEtot = 0.0;
       nPhotons_ps = 0;
       for(unsigned int i = 0; i < photons.size() ; i++) {
	  if ( photons[i]->getTag() == DPhoton::kCharge  ) continue;
	  nPhotons_ps++;
	  neutEtot+= photons[i]->lorentzMomentum().T();
	  
       }
       // cout << photons.size() << " "<< recoilproton.size() << endl;
       

       ProbMax = 0.0;
       ProbMax_PI0 = 0.0;
       ProbMax_ETA = 0.0;

       PULL0=0;
       PULL1=0;
       PULL2=0;
       PULL3=0;
       PULL4=0;

       ETACHI2 = 0.0;
       PI0CHI2 = 0.0;
    
       nPairs = 0;
       PI0MASS = 0.0;
       ETAMASS = 0.0;
 
       RECMASS = 0.0;
       RECENERGY= 0.0;
       RECMOM = 0.0;
       RECPT = 0.0;


       RESMASS = 0.0;
       RESENERGY= 0.0;
       MISMASS = 0.0;
       MISENERGY = 0.0;
       
       RESPT = 0.0;
       TOTALPT = 0.0;
       float Prob = 0.0;
       ProbMax = 0.0;
       cosGJ=0.0;
       phiGJ=0.0;
       t=0.0;
       nPart=2;
       nCandChi2_3 = 0;
       nCandChi2_5 = 0;
       nFitCandChi2_50 = 0;
       nFitCandProb = 0;
        DMatrixDSym dumerr(7);
       
       for (int np = 0; np<nPart;np++){
       m_e[np] = 0.0;
       m_px[np] = 0.0;
       m_py[np] = 0.0;
       m_pz[np] = 0.0;
       }

       lowresphot_energy = 20.0;
       lowresphot_theta = 20.0;
       for (unsigned int zero = 0; zero<4;zero++){
	 resphot_energy[zero] = 0.0;
	 resphot_theta[zero] = 0.0;
       }
      
       

       RECMASS =0;
       t = 0;
         
       m_eRecoil = 0.0;
       m_pxRecoil = 0.0;
       m_pyRecoil = 0.0;
       m_pzRecoil = 0.0;




       TLorentzVector P4_beam( 0.0, 0.0, 9.0, 9.0 );       
       m_eBeam = P4_beam.T();
       m_pxBeam = P4_beam.X();
       m_pyBeam = P4_beam.Y();
       m_pzBeam = P4_beam.Z();

       TLorentzVector P4_target( 0, 0, 0, 0.938 );

       TLorentzVector P4_RECOIL(0,0,0,0);



       //     DKinematicData kd_beam = DKinematicData();
       //	DKinematicData kd_target = DKinematicData();
       //	vector<DKinematicData> kd_initialState;
       //	vector<DKinematicData> kd_finalState;
       //	vector<int> constraintParticles;
       //	vector<DKinematicData> kindata_PI0;
       //	vector<DKinematicData> kinout_PI0 ;
       //	DVector3 vertex_PI0;
       
       //	vector<DKinematicData> kindata_ETA;
       //	vector<DKinematicData> kinout_ETA;
       //	DVector3 vertex_ETA;
	

     
          if( recoilproton.size() == 1 || recoilproton.size() == 0) {
       //      if( recoilproton.size() == 1){
       //        JObject::oid_t nPairs=0;
	// Loop over all fit candidates.
       if ( nPhotons_ps >= 4 &&  nPhotons_ps <= 8){
        for(unsigned int i = 0; i < photons.size() ; i++) {
            if ( photons[i]->getTag() == DPhoton::kCharge  ) continue;

            for (unsigned int j = i+1; j < photons.size() ; j++) {
                if ( photons[j]->getTag() == DPhoton::kCharge  ) continue;

		 for (unsigned int k = 0; k < photons.size() ; k++) {
		   if ( photons[k]->getTag() == DPhoton::kCharge ||  k == i || k == j ) continue;

		   for (unsigned int l = k+1; l < photons.size() ; l++) {
		     if ( photons[l]->getTag() == DPhoton::kCharge || l == i || l == j )continue;
		    
		     //[i,j] = pair of photons for pions, [k,l] = pair of photons for eta
		     //	cout << "pi0:[" << i << "," << j << "] eta:[" << k << "," << l << "]" <<endl;







		////////////////////////////////
		///Fit Initial and Final States
		////////////////////////////////

		   
		     dumerr(0,0) = 1.0;
		     //cerr << "FILLING ERROR MATRIX" << endl;
		     //cerr << "dum row/col: " << dumerr.GetNrows() << " " << dumerr.GetNcols() << endl;
		     //cerr << "kin row/col: " << _kDataInitial_out[0].errorMatrix().GetNrows() << " " << _kDataInitial_out[0].errorMatrix().GetNcols() << endl;
		     //_kDataInitial_out[0].setErrorMatrix(dumerr);
		     //cerr << "FILLED ERROR MATRIX" << endl;
		
		
		     vector<DKinematicData> kd_initialState; //initial state kinematics

     
		     DKinematicData kd_beam = DKinematicData();//beam kinematics
		kd_beam.setMass(0.0);
		kd_beam.setCharge(0);
		kd_beam.setMassFixed();
		kd_beam.setMomentum( TVector3(0.0, 0.0, 9.0));
		kd_beam.clearErrorMatrix(); // set error matrix with entries equal to zero
		kd_initialState.push_back(kd_beam);
		kd_initialState[0].smearMCThrownMomentum(0.001);

		DKinematicData kd_target = DKinematicData();  //target kinematics
		kd_target.setMass(0.93827);
		kd_target.setCharge(1);
		kd_target.setMassFixed();
		kd_target.setMomentum( TVector3(0.0, 0.0, 0.0));
		kd_target.clearErrorMatrix();  // set error matrix with entries equal to zero
		kd_initialState.push_back(kd_target);
		kd_initialState[1].smearMCThrownMomentum(0.00001);


	
			vector<DKinematicData> kd_finalState; //final state kinematics
		 //build the 5 particles in the fit

		
			kd_finalState.push_back(*(photons[i]));
		
			kd_finalState.push_back(*(photons[j]));
		
			kd_finalState.push_back(*(photons[k]));
		
			kd_finalState.push_back(*(photons[l]));

			//	 kd_finalState.push_back(*(photons[j]));
			// kd_finalState.push_back(*(photons[k]));
			// kd_finalState.push_back(*(photons[l]));
		 if( recoilproton.size() == 1  ) {
		   P4_RECOIL = recoilproton[0]->lorentzMomentum();
		   kd_finalState.push_back(*(recoilproton[0]));
		   // kd_finalState[4].clearErrorMatrix();
		
		 }
    	
		 

		 DKinFit *kfit = new DKinFit(); 
		 kfit->ResetForNewFit();
		 kfit->SetVerbose(0);
		 kfit->UseTimingInformation();
		 
		 kfit->SetInitial(kd_initialState);
		 kfit->SetFinal(kd_finalState);
		 if(recoilproton.size() == 0) {
		   //   cout << "NPart= = " <<recoilproton.size() << endl;
		   kfit->SetMissingParticle(0.938);

		 }
		 	 vector<int> constraintParticles;
		 constraintParticles.push_back(0);
		 constraintParticles.push_back(1);
	       	 kfit->SetMassConstraint(0.13498,constraintParticles); //constrains first two photons to the pi0 mass

		 constraintParticles.clear();
		 constraintParticles.push_back(2);
		 constraintParticles.push_back(3);
		 kfit->SetMassConstraint(0.54775,constraintParticles); //constrains the second two particles to the eta mass

		 kfit->Fit();  // the fit is actually done here

		 //get the fit chi2 and probabily and pulls
		 FitProb[nPairs] = kfit->Prob();
		 FitChi2[nPairs] = kfit->Chi2();
		 FitNDF[nPairs] = kfit->Ndf();

		 for(unsigned int pullk = 0; pullk<15;pullk++){
		 FitPull[pullk][nPairs] = kfit->GetPull(pullk+6);
		 }

		 CovMat_FIT = kfit->GetCovMat();

		 //	 FitPull0[nPairs] = kfit->GetPull(6);
		 //	 FitPull1[nPairs] = kfit->GetPull(7);
		 //	 FitPull2[nPairs] = kfit->GetPull(8);
		 //	 FitPull3[nPairs] = kfit->GetPull(9);
		 //	 FitPull4[nPairs] = kfit->GetPull(10);
		 
		 //	 cout << FitPull0[nPairs] << " " << kfit->GetPull(6) << " " <<  kfit->Prob() << endl;
		  //fitter has adjusted the 4-momenta of the particles within error. get these new values.
		 vector<DKinematicData> kinout = kfit->GetFinal_out();
		 vector<DKinematicData> kinin = kfit->GetFinal_in();
		 
		 delete kfit;
		 TLorentzVector P4_PI0,P4_ETA,P4_REC,P4_RES,P4_MISS4M;		
      		 P4_PI0 = kinout[0].lorentzMomentum() + kinout[1].lorentzMomentum(); 
		 P4_ETA = kinout[2].lorentzMomentum() + kinout[3].lorentzMomentum();
	 if( recoilproton.size() == 1  ) {
	   P4_RECOIL = kinout[4].lorentzMomentum();
	   P4_MISS4M = P4_beam + P4_target - P4_ETA - P4_PI0 -P4_RECOIL;
	   
	 }
	 else {
	   P4_RECOIL = P4_beam + P4_target - P4_ETA - P4_PI0;
	   P4_MISS4M = P4_RECOIL;
	 }


	     P4_MISS4M = P4_beam + P4_target - P4_ETA - P4_PI0 -P4_RECOIL;
       MISMASS = P4_MISS4M.M();
       MISMASS2 = P4_MISS4M.M2();

       MISENERGY = P4_MISS4M.T();
       MISMOM = P4_MISS4M.Vect().Mag();
       MISPT = P4_MISS4M.Pt();
     
       RECMASS = P4_RECOIL.M();
       RECENERGY = P4_RECOIL.T();
       RECMOM = P4_RECOIL.Vect().Mag();
       RECPT = P4_RECOIL.Pt();
       

 

       if ( P4_RECOIL.Beta()>0 &&  P4_RECOIL.Beta()<1 ){
       m_eRecoil = P4_RECOIL.T();
       m_pxRecoil = P4_RECOIL.X();
       m_pyRecoil = P4_RECOIL.Y();
       m_pzRecoil = P4_RECOIL.Z();
       }

	       
		gamma4_pt[nPairs] = sqrt( ( P4_ETA.X() + P4_PI0.X() ) * ( P4_ETA.X() + P4_PI0.X() ) + ( P4_ETA.Y() + P4_PI0.Y() ) * ( P4_ETA.Y() + P4_PI0.Y() ));
		total_pt[nPairs] = sqrt( ( P4_ETA.X() + P4_PI0.X() + m_pxRecoil ) * ( P4_ETA.X() + P4_PI0.X() +m_pxRecoil ) + ( P4_ETA.Y() + P4_PI0.Y() + m_pyRecoil) * ( P4_ETA.Y() + P4_PI0.Y() + m_pyRecoil ));

		//		cout << "tot: " << total_pt[nPairs] << endl;;
		
		//		cout << nPairs << " "  << eta_mass[nPairs] << endl;




		  /// photons in the pion rest frame
		  TLorentzRotation PI0RestBoost_com( -P4_PI0.BoostVector() );
		  //		  DLorentzVector beam_res   = resRestBoost * P4_beam;
		  //		  DLorentzVector recoil_res = resRestBoost * P4_RECOIL;
		  DLorentzVector pi0_g1_com = PI0RestBoost_com * kinin[0].lorentzMomentum();
		  DLorentzVector pi0_g2_com = PI0RestBoost_com * kinin[1].lorentzMomentum();

    
		  DVector3 pi0z_com = P4_PI0.Vect().Unit();
		  DVector3 pi0y_com = pi0_g1_com.Vect().Cross(pi0z_com).Unit();
		  DVector3 pi0x_com= pi0y_com.Cross(pi0z_com);

		  
		  DVector3 angles_g1_com( (pi0_g1_com.Vect()).Dot(pi0x_com),
		     (pi0_g1_com.Vect()).Dot(pi0y_com),
		     (pi0_g1_com.Vect()).Dot(pi0z_com) );

		  DVector3 angles_g2_com( (pi0_g2_com.Vect()).Dot(pi0x_com),
		     (pi0_g2_com.Vect()).Dot(pi0y_com),
		     (pi0_g2_com.Vect()).Dot(pi0z_com) );



		  pi0theta_g1_com[nPairs] = angles_g1_com.Theta(); 
		  pi0phi_g1_com[nPairs] = angles_g1_com.Phi();
		  pi0theta_g2_com[nPairs] = angles_g2_com.Theta(); 
		  pi0phi_g2_com[nPairs] = angles_g2_com.Phi();
		  pi0e_g1_com[nPairs] = pi0_g1_com.T();
		  pi0e_g2_com[nPairs] = pi0_g2_com.T();
		  ///////////////////////////////////////

		  /// photons in the eta rest frame
		  TLorentzRotation ETARestBoost_com( -P4_ETA.BoostVector() );
		  //		  DLorentzVector beam_res   = resRestBoost * P4_beam;
		  //		  DLorentzVector recoil_res = resRestBoost * P4_RECOIL;
		  DLorentzVector eta_g1_com = ETARestBoost_com * kinin[2].lorentzMomentum();
		  DLorentzVector eta_g2_com = ETARestBoost_com * kinin[3].lorentzMomentum();

    
		  DVector3 etaz_com = P4_ETA.Vect().Unit();
		  DVector3 etay_com = eta_g1_com.Vect().Cross(etaz_com).Unit();
		  DVector3 etax_com= etay_com.Cross(etaz_com);

		  
		  DVector3 angles_eta_g1_com( (eta_g1_com.Vect()).Dot(etax_com),
		     (eta_g1_com.Vect()).Dot(etay_com),
		     (eta_g1_com.Vect()).Dot(etaz_com) );

		  DVector3 angles_eta_g2_com( (eta_g2_com.Vect()).Dot(etax_com),
		     (eta_g2_com.Vect()).Dot(etay_com),
		     (eta_g2_com.Vect()).Dot(etaz_com) );



		  etatheta_g1_com[nPairs] = angles_eta_g1_com.Theta(); 
		  etaphi_g1_com[nPairs] = angles_eta_g1_com.Phi();
		  etatheta_g2_com[nPairs] = angles_eta_g2_com.Theta(); 
		  etaphi_g2_com[nPairs] = angles_eta_g2_com.Phi();
		  etae_g1_com[nPairs] = eta_g1_com.T();
		  etae_g2_com[nPairs] = eta_g2_com.T();
		  ///////////////////////////////////////










		
		if (FitChi2[nPairs]/FitNDF[nPairs] < 10) nFitCandChi2_50++; //counts the candidates
		if (FitProb[nPairs] > 0.01) {

		  nFitCandProb++; //counts the candidates
		  write_out = true;
		}

		//if the combination has the maximum probability choose this combination as the candidate
		if (FitProb[nPairs] > ProbMax){
		  ProbMax = FitProb[nPairs];

	
		  ProbMax_FIT = FitProb[nPairs];
		  FITCHI2 = FitChi2[nPairs];
		  FITNDF = FitNDF[nPairs];

		  for(unsigned int pullk = 0; pullk < 15; pullk++){
		  PULL[pullk] = FitPull[pullk][nPairs];
		  }
		  CovMat = CovMat_FIT;

		  //  cerr << "Total error matrix: " << CovMat_FIT.GetNrows() << " by " << CovMat_FIT.GetNcols() << endl;
		  //	  for (int nrows = 0; nrows < CovMat.GetNrows(); nrows++ ){
		  //	    for (int ncols = 0; ncols < CovMat.GetNcols(); ncols++ ){
		  //	      cerr << CovMat(nrows,ncols) << " ";

		  //    }
		  //   cerr << endl;
		  //  }
		  
		  // PULL1 = FitPull1[nPairs];
		  // PULL2 = FitPull2[nPairs];
		  // PULL3 = FitPull3[nPairs];
		  // PULL4 = FitPull4[nPairs];
		  //	  cout << PULL[0] << " " << PULL[1] << " " << PULL[2] << " " << PULL[3] << " " << PULL[4] << " " << PULL[5] << " " << PULL[6] << " " <<  kfit->Prob() << endl;

		  PI0MASS = P4_PI0.M();
		  ETAMASS = P4_ETA.M();

		  P4_RES = P4_PI0 + P4_ETA;
		  RESMASS = P4_RES.M();
		  RESENERGY = P4_RES.T();
		  RESMOM =  P4_RES.Vect().Mag();
		  RESPT = P4_RES.Pt();
                  t = -( P4_beam - P4_RES ).Mag2();

		  /// photons in the pion rest frame
		  TLorentzRotation PI0RestBoost( -P4_PI0.BoostVector() );
		  //		  DLorentzVector beam_res   = resRestBoost * P4_beam;
		  //		  DLorentzVector recoil_res = resRestBoost * P4_RECOIL;
		  DLorentzVector pi0_g1 = PI0RestBoost * kinin[0].lorentzMomentum();
		  DLorentzVector pi0_g2 = PI0RestBoost * kinin[1].lorentzMomentum();

    
		  DVector3 pi0z = P4_PI0.Vect().Unit();
		  DVector3 pi0y = pi0_g1.Vect().Cross(pi0z).Unit();
		  DVector3 pi0x = pi0y.Cross(pi0z);

		  
		  DVector3 angles_g1( (pi0_g1.Vect()).Dot(pi0x),
		     (pi0_g1.Vect()).Dot(pi0y),
		     (pi0_g1.Vect()).Dot(pi0z) );

		  DVector3 angles_g2( (pi0_g2.Vect()).Dot(pi0x),
		     (pi0_g2.Vect()).Dot(pi0y),
		     (pi0_g2.Vect()).Dot(pi0z) );



		  pi0theta_g1 = angles_g1.Theta(); 
		  pi0phi_g1 = angles_g1.Phi();
		  pi0theta_g2 = angles_g2.Theta(); 
		  pi0phi_g2 = angles_g2.Phi();
		  pi0e_g1 = pi0_g1.T();
		  pi0e_g2 = pi0_g2.T();
		  
		  ///////////////////////////////////////


		  /// photons in the pion rest frame
		  TLorentzRotation ETARestBoost( -P4_ETA.BoostVector() );
		  //		  DLorentzVector beam_res   = resRestBoost * P4_beam;
		  //		  DLorentzVector recoil_res = resRestBoost * P4_RECOIL;
		  DLorentzVector eta_g1 = ETARestBoost * kinin[2].lorentzMomentum();
		  DLorentzVector eta_g2 = ETARestBoost * kinin[3].lorentzMomentum();

    
		  DVector3 etaz = P4_ETA.Vect().Unit();
		  DVector3 etay = eta_g1.Vect().Cross(etaz).Unit();
		  DVector3 etax = etay.Cross(etaz);

		  
		  DVector3 angles_eta_g1( (eta_g1.Vect()).Dot(etax),
		     (eta_g1.Vect()).Dot(etay),
		     (eta_g1.Vect()).Dot(etaz) );

		  DVector3 angles_eta_g2( (eta_g2.Vect()).Dot(etax),
		     (eta_g2.Vect()).Dot(etay),
		     (eta_g2.Vect()).Dot(etaz) );



		  etatheta_g1 = angles_eta_g1.Theta(); 
		  etaphi_g1 = angles_eta_g1.Phi();
		  etatheta_g2 = angles_eta_g2.Theta(); 
		  etaphi_g2 = angles_eta_g2.Phi();
		  etae_g1 = eta_g1.T();
		  etae_g2 = eta_g2.T();
		  
		  ///////////////////////////////////////












		  
		  TOTALPT = total_pt[nPairs];
		  //		  cout <<" TOT: "<< TOTALPT << endl;
		  
		  resphot_energy[0] = kinout[0].lorentzMomentum().T();
		  resphot_energy[1] = kinout[1].lorentzMomentum().T();
		  resphot_energy[2] = kinout[2].lorentzMomentum().T();
		  resphot_energy[3] = kinout[3].lorentzMomentum().T();

		  resphot_theta[0] = kinout[0].lorentzMomentum().Vect().Theta();
		  resphot_theta[1] = kinout[1].lorentzMomentum().Vect().Theta();
		  resphot_theta[2] = kinout[2].lorentzMomentum().Vect().Theta();
		  resphot_theta[3] = kinout[3].lorentzMomentum().Vect().Theta();

		  etapair_energydiff =  resphot_energy[2] -  resphot_energy[3];
		  pi0pair_energydiff =  resphot_energy[0] -  resphot_energy[1];

		  resphot_in_energy[0] = kinin[0].lorentzMomentum().T();
		  resphot_in_energy[1] = kinin[1].lorentzMomentum().T();
		  resphot_in_energy[2] = kinin[2].lorentzMomentum().T();
		  resphot_in_energy[3] = kinin[3].lorentzMomentum().T();

		  resphot_in_theta[0] = kinin[0].lorentzMomentum().Vect().Theta();
		  resphot_in_theta[1] = kinin[1].lorentzMomentum().Vect().Theta();
		  resphot_in_theta[2] = kinin[2].lorentzMomentum().Vect().Theta();
		  resphot_in_theta[3] = kinin[3].lorentzMomentum().Vect().Theta();


		  if (nFitCandProb>0)

		  
		  for (int small = 0;small <4;small++){
		    if (lowresphot_energy >resphot_energy[small]) {
		      lowresphot_energy = resphot_energy[small];
		      lowresphot_theta = resphot_theta[small];
		      lowresphot_in_energy = resphot_in_energy[small];
		      lowresphot_in_theta = resphot_in_theta[small];
		    }
		  }
		  

		  ///fills variables for the kin tree

    if ( (P4_RES.Beta() > 0 && P4_RES.Beta() < 1) && P4_ETA.T() < 10 && P4_PI0.T() < 10 ) {
		  
		  TLorentzRotation resRestBoost( -P4_RES.BoostVector() );

		  m_e[0] = P4_ETA.T();
		  m_e[1] = P4_PI0.T();
		  		
		  m_px[0] = P4_ETA.X();
		  m_px[1] = P4_PI0.X();
		  
		  m_py[0] = P4_ETA.Y();
		  m_py[1] = P4_PI0.Y();

		  m_pz[0] = P4_ETA.Z();
		  m_pz[1] = P4_PI0.Z();

		  

    DLorentzVector beam_res   = resRestBoost * P4_beam;
    DLorentzVector recoil_res = resRestBoost * P4_RECOIL;
    DLorentzVector p1_res = resRestBoost * P4_PI0;

    
    DVector3 z = beam_res.Vect().Unit();
    DVector3 y = recoil_res.Vect().Cross(z).Unit();
    DVector3 x = y.Cross(z);

   
    DVector3 angles( (p1_res.Vect()).Dot(x),
		     (p1_res.Vect()).Dot(y),
		     (p1_res.Vect()).Dot(z) );

    cosGJ = angles.CosTheta(); 
    phiGJ = angles.Phi();
   
    }
   
		}
		nPairs++;
	 
      		
                 
		   }         //end of "for (unsigned int l...){"  
		 }         //end of "for (unsigned int j...){"  
	    }         //end of "for (unsigned int j...){"  
        } //end of "for (unsigned int i...){"  
       } //end of "if (nPhotons_ps >= 4...){"
    }  //end of "if( recoilproton.size() == 1 && ...){"

 


       if ( m_e[0] > 10 || m_e[1] > 10 )  cout << "BROKEN!!!" << endl;
		  		  

#endif

if( write_out ){

    flush_s_HDDM(hddm, file);
    Nevents_written++;
  }

 hddm_source->flush_on_free = !write_out;


        m_outTree->Fill();
        m_tree->Fill();
       
	return NOERROR;
}

//------------------
// erun
//------------------
jerror_t FCALTreeProc::erun(void)
{
	// Any final calculations on histograms (like dividing them)
	// should be done here. This may get called more than once.
	return NOERROR;
}

//------------------
// fini
//------------------
jerror_t FCALTreeProc::fini(void)
{

	if(file){
		close_s_HDDM(file);
		cout<<endl<<"Closed HDDM file"<<endl;
	}
	cout<<" "<<Nevents_written<<" events written to "<<filename<<endl;
	// If another JEventProcessor is in the list ahead of this one, then
	// it will have finished before this is called. e.g. closed the
	// ROOT file!
	
	m_treeFile->Write();
	m_treeFile->Close();
		cout<<endl<<"Closed ROOT file"<<endl;
	return NOERROR;
}

