// $Id$
//
//    File: FCALClusterProc.h
// Created: Wed Aug 10 13:17:20 EST 2005
// Creator: shepherd (on Darwin 129-79-159-16.dhcp-bl.indiana.edu 7.9.0 powerpc)
//

#ifndef _FCALTreeProc_
#define _FCALTreeProc_

#include <JANA/JEventProcessor.h>
#include <JANA/JEventLoop.h>
//#include <JANA/JFactory.h>
using namespace jana;

#include "TFile.h"
#include "TTree.h"
#include "TVector3.h"
#include "DLorentzVector.h"
#include <vector>
#include "TClonesArray.h"
#include "PID/DTwoGammaFit_factory.h"
#include "TMatrixD.h"
#include "HDDM/hddm_s.h"

#define MC_THROWN_MAX 100
#define FCAL_TRUTH_MAX 500
#define FCAL_HITS_MAX 2800
#define FCAL_CLUST_MAX 20
#define BCAL_CLUST_MAX 50
#define PID_PHOT_MAX 70
#define PID_PART_MAX 70
#define PID_2GAMMA_MAX 2415
#define BCAL_TRUTH_MAX 500

// * Structure to hold Monte Carlo Thrown Data
typedef struct {
  int type;		// type 
  float q;		// charge
  float p;		// momentum
  double E;		// energy
  double the;		// polar angle
  double phi;		// azimuth
  float M;		// invariant mass
  float vertex[3]; 		// vertex of thrown particle
} MCThrown_t ;




class FCALTreeProc:public JEventProcessor{
	public:
		//FCALTreeProc(){};
		//~FCALTreeProc(){};
		//const char* className(void){return "FCALTreeProc";}
                inline DLorentzVector lorentzMomentum() const { return Gtot; }
		string filename;
		s_iostream_t *file;
		unsigned long Nevents_written;
		
		bool write_out;


	private:
		jerror_t init(void);			///< Called once at program start.
		jerror_t brun(JEventLoop *eventLoop, int runnumber);	///< Called everytime a new run number is detected.
		jerror_t evnt(JEventLoop *eventLoop, int eventnumber);	///< Called every event.
		jerror_t erun(void);			///< Called everytime run number changes, provided brun has been called.
		jerror_t fini(void);				///< Called after last event of last event source has been processed.

// Info in DMCThrown from pythia
//		int type;			///< GEANT particle ID
//		int pdgtype;		///< PDG particle type (not used by GEANT)
//		int myid;			///< id of this particle from original generator
//		int parentid;		///< id of parent of this particle from original generator
//		int mech;			///< production mechanism of this partcle (generator specific)

		TFile* m_treeFile;
                TTree* m_tree;
		TTree* m_outTree;

  //thrown information
  //// Declaration of leaf types

                int nThrown;
		int TnPhotons;
		int TnProtons;
		int nCharged;
                int thrownType[MC_THROWN_MAX];
                int thrownTypePDG[MC_THROWN_MAX];
                int thrownMID[MC_THROWN_MAX];
                int thrownParent[MC_THROWN_MAX];
                float thrownEnergy[MC_THROWN_MAX];
                float thrownEnergyTot;
		float thrownNeutEnergyTot;
                float thrownP[MC_THROWN_MAX];
                float thrownMass[MC_THROWN_MAX];
                float thrownPhi[MC_THROWN_MAX];
                float thrownTheta[MC_THROWN_MAX];
                float thrownQ[MC_THROWN_MAX];
                float thrownVx[MC_THROWN_MAX];
                float thrownVy[MC_THROWN_MAX];
                float thrownVz[MC_THROWN_MAX];

		float thrownlowresphot_energy;
		float thrownlowresphot_theta;
		float thrownResMass;
		

		 
		float TFitProb[PID_2GAMMA_MAX];
		float TFitChi2[PID_2GAMMA_MAX];
		float TFitNDF[PID_2GAMMA_MAX];

		float TFitPull0[PID_2GAMMA_MAX];
		float TFitPull1[PID_2GAMMA_MAX];
		float TFitPull2[PID_2GAMMA_MAX];
		float TFitPull3[PID_2GAMMA_MAX];
		float TFitPull4[PID_2GAMMA_MAX];

		DTwoGammaFit* Tfit2g_PI0;
		DTwoGammaFit* Tfit2g_ETA;

		float Teta_mass[PID_2GAMMA_MAX];
		float Teta_prob[PID_2GAMMA_MAX];
		float Teta_Chi2[PID_2GAMMA_MAX];
		float Teta_pull_1[PID_2GAMMA_MAX];
		float Teta_pull_2[PID_2GAMMA_MAX];
		float Teta_pull_3[PID_2GAMMA_MAX];
		float Teta_pull_4[PID_2GAMMA_MAX];
		float Teta_pull_5[PID_2GAMMA_MAX];
		float Teta_pull_6[PID_2GAMMA_MAX];
		float Teta_momentum_Mag[PID_2GAMMA_MAX];
		float Teta_energy[PID_2GAMMA_MAX];

		int TnPairs;


		float Tpi0_mass[PID_2GAMMA_MAX];
		float Tpi0_prob[PID_2GAMMA_MAX];
		float Tpi0_Chi2[PID_2GAMMA_MAX];
		float Tpi0_pull_1[PID_2GAMMA_MAX];
		float Tpi0_pull_2[PID_2GAMMA_MAX];
		float Tpi0_pull_3[PID_2GAMMA_MAX];
		float Tpi0_pull_4[PID_2GAMMA_MAX];
		float Tpi0_pull_5[PID_2GAMMA_MAX];
		float Tpi0_pull_6[PID_2GAMMA_MAX];
		float Tpi0_momentum_Mag[PID_2GAMMA_MAX];
		float Tpi0_energy[PID_2GAMMA_MAX];

		float Tgamma4_energy[PID_2GAMMA_MAX];
		float Tgamma4_pt[PID_2GAMMA_MAX];
		float Ttotal_pt[PID_2GAMMA_MAX];


		
		float TProbMax;
		float TProbMax_PI0;
	        float TProbMax_ETA;
	        float TProbMax_FIT;

		float TPI0MASS;
		float TETAMASS;
		float TPI0CHI2;
		float TETACHI2;
		float TFITCHI2;
		float TFITNDF;


  
		float TRECMASS;
		float TRESMASS;
		float TMISMASS;
 
		float TRECMOM;
		float TRESMOM;
		float TMISMOM;
		
		float TRESENERGY;
		float TRECENERGY;
		float TMISENERGY;
		
		float TRECPT;
		float TRESPT;
		float TMISPT;
		float TTOTALPT;
		
		float Tt;
		float TcosGJ;
		float TphiGJ;

      
        int Tm_eventCounter;
	int TnCandChi2_3;
	int TnCandChi2_5;
	int TnFitCandChi2_10;
	

        int TnPart;
		

// TruthShowers in FCAL
                int nFTruth;
                //int fTruthTrack[FCAL_TRUTH_MAX];
                int fTSPrim[FCAL_TRUTH_MAX];
                float fTruthE[FCAL_TRUTH_MAX];
                float fTruthT[FCAL_TRUTH_MAX];
                float fTruthX[FCAL_TRUTH_MAX];
                float fTruthY[FCAL_TRUTH_MAX];
                float fTruthZ[FCAL_TRUTH_MAX];

// hits in FCAL
                int nFcal;
                float fcalEtot;
                float fcalE[FCAL_HITS_MAX];
                float fcalT[FCAL_HITS_MAX];
                float fcalX[FCAL_HITS_MAX];
                float fcalY[FCAL_HITS_MAX];
                float fcalR[FCAL_HITS_MAX];
                float fcalC[FCAL_HITS_MAX];
                float fcalRg[FCAL_HITS_MAX];
                float fcalCg[FCAL_HITS_MAX];
                float fcalS;
		
// clusters in FCAL
                int nFcalClust;
                float fcalClustEtot;
                float fcalClustE[FCAL_CLUST_MAX];
                float fcalClustX[FCAL_CLUST_MAX];
                float fcalClustY[FCAL_CLUST_MAX];
                float fcalClustZ[FCAL_CLUST_MAX];
//                float fcalClustT[FCAL_CLUST_MAX];
                float fcalClustS[FCAL_CLUST_MAX];  // cluster size
                float fcalClustSu[FCAL_CLUST_MAX]; // longitudinal
                float fcalClustSv[FCAL_CLUST_MAX]; // transverse size

// photons in FCAL
                int nFcalPhot;
                float fcalPhotEtot;
                float fcalPhotMtot;
	       
                float fcalPhotM[FCAL_CLUST_MAX];
                float fcalPhotPx[FCAL_CLUST_MAX];
                float fcalPhotPy[FCAL_CLUST_MAX];
                float fcalPhotPz[FCAL_CLUST_MAX];
                float fcalPhotE[FCAL_CLUST_MAX];
                float fcalPhotX[FCAL_CLUST_MAX];
                float fcalPhotY[FCAL_CLUST_MAX];
                float fcalPhotZ[FCAL_CLUST_MAX];
                float fcalPhotXerr[FCAL_CLUST_MAX];
                float fcalPhotYerr[FCAL_CLUST_MAX];
                float fcalPhotZerr[FCAL_CLUST_MAX];
                float fcalPhotT[FCAL_CLUST_MAX];
 
// clusters in BCAL IU
                int nBcalClust;
                float bcalClustEtot;
                float bcalClustE[BCAL_CLUST_MAX];
                float bcalClustX[BCAL_CLUST_MAX];
                float bcalClustY[BCAL_CLUST_MAX];
                float bcalClustZ[BCAL_CLUST_MAX];
                float bcalClustT[BCAL_CLUST_MAX];
                float bcalClustRho[BCAL_CLUST_MAX];
                float bcalClustPhi[BCAL_CLUST_MAX];
                float bcalClustThe[BCAL_CLUST_MAX];

// photons in BCAL (Matt)
                int nBcalPhot;
                float bcalPhotEtot;
                //float bcalPhotMtot;
                float bcalPhotE[BCAL_CLUST_MAX];
                float bcalPhotX[BCAL_CLUST_MAX];
                float bcalPhotY[BCAL_CLUST_MAX];
                float bcalPhotZ[BCAL_CLUST_MAX];
                float bcalPhotT[BCAL_CLUST_MAX];
                float bcalPhotPx[BCAL_CLUST_MAX];
                float bcalPhotPy[BCAL_CLUST_MAX];
                float bcalPhotPz[BCAL_CLUST_MAX];
                float bcalPhotAx[BCAL_CLUST_MAX];
                float bcalPhotAy[BCAL_CLUST_MAX];
                float bcalPhotAz[BCAL_CLUST_MAX];
                float bcalPhotEAx[BCAL_CLUST_MAX];
                float bcalPhotEAy[BCAL_CLUST_MAX];
                float bcalPhotEAz[BCAL_CLUST_MAX];
// TruthShowers in BCAL
                int nBTruth;
                int bTruthTrack[BCAL_TRUTH_MAX];
                int bTSPrim[BCAL_TRUTH_MAX];
                float bTruthE[BCAL_TRUTH_MAX];
                float bTruthT[BCAL_TRUTH_MAX];
                float bTruthRho[BCAL_TRUTH_MAX];
                float bTruthPhi[BCAL_TRUTH_MAX];
                float bTruthZ[BCAL_TRUTH_MAX];
//PID
		DLorentzVector Gtot;

// PID photons 
                int nPhot;
                float photEtot;
                float photMtot;
                float photPttot;
                float photE[PID_PHOT_MAX];
                float photPx[PID_PHOT_MAX];
                float photPy[PID_PHOT_MAX];
                float photPz[PID_PHOT_MAX];
                float photX[PID_PHOT_MAX];
                float photY[PID_PHOT_MAX];
                float photZ[PID_PHOT_MAX];
                float photThe[PID_PHOT_MAX];
                float photPhi[PID_PHOT_MAX];
                float photDtRT[PID_PHOT_MAX];
                float photErr00[PID_PHOT_MAX];
                float photErr11[PID_PHOT_MAX];
                float photErr22[PID_PHOT_MAX];
                float photErr33[PID_PHOT_MAX];
                float photErr44[PID_PHOT_MAX];
                float photErr55[PID_PHOT_MAX];
                float photErr66[PID_PHOT_MAX];
                int photTag[PID_PHOT_MAX];

		int nPhotC_bcal;
		int nPhotC_fcal;
		TVector3 *_bcal_phot_calPos;
		TVector3 *_fcal_phot_calPos;

		float bcal_phot_diffmin_Mag[PID_PHOT_MAX];
		float fcal_phot_diffmin_Mag[PID_PHOT_MAX]; 
		float bcal_phot_diffmin_X[PID_PHOT_MAX];
		float fcal_phot_diffmin_X[PID_PHOT_MAX]; 
		float bcal_phot_diffmin_Y[PID_PHOT_MAX];
		float fcal_phot_diffmin_Y[PID_PHOT_MAX]; 
		float bcal_phot_diffmin_Z[PID_PHOT_MAX];
		float fcal_phot_diffmin_Z[PID_PHOT_MAX]; 
		float fcal_phot_diffmin_Phi[PID_PHOT_MAX]; 
		float bcal_phot_diffmin_Phi[PID_PHOT_MAX]; 
		int bcal_phot_diffTag[PID_PHOT_MAX];
		int fcal_phot_diffTag[PID_PHOT_MAX]; 

		TClonesArray *array;


// PID particles 
                int nPart_p;
                float partEtot;
                float partMtot;
		float partPttot;
                float partE[PID_PART_MAX];
                float partPx[PID_PART_MAX];
                float partPy[PID_PART_MAX];
                float partPz[PID_PART_MAX];
                float bcal_partPx[PID_PART_MAX];
                float bcal_partPy[PID_PART_MAX];
                float bcal_partPz[PID_PART_MAX];
                float fcal_partPx[PID_PART_MAX];
                float fcal_partPy[PID_PART_MAX];
                float fcal_partPz[PID_PART_MAX];
                float bcal_partX[PID_PART_MAX];
                float bcal_partY[PID_PART_MAX];
                float bcal_partZ[PID_PART_MAX];
                float fcal_partX[PID_PART_MAX];
                float fcal_partY[PID_PART_MAX];
                float fcal_partZ[PID_PART_MAX];
		TVector3 *_partMom;
		TVector3 *_bcal_swum_partPos;
		TVector3 *_bcal_swum_partMom;
		TVector3 *_fcal_swum_partPos;
		TVector3 *_fcal_swum_partMom;

// PID two gamma fit
                int fPi0;
                float fPi0Mtot;
                int fPi0ID[PID_2GAMMA_MAX];
                float fPi0M[PID_2GAMMA_MAX];
                float fPi0E[PID_2GAMMA_MAX];
                float fPi0P[PID_2GAMMA_MAX];
                float fPi0The[PID_2GAMMA_MAX];
                float fPi0Phi[PID_2GAMMA_MAX];
                float fPi0Prob[PID_2GAMMA_MAX];
                float fPi0Chi2[PID_2GAMMA_MAX];
                float fPi0Ndf[PID_2GAMMA_MAX];
                float fPi0Pul0[PID_2GAMMA_MAX];
                float fPi0Pul1[PID_2GAMMA_MAX];
                float fPi0Pul2[PID_2GAMMA_MAX];
                float fPi0Pul3[PID_2GAMMA_MAX];
                float fPi0Pul4[PID_2GAMMA_MAX];
                float fPi0Pul5[PID_2GAMMA_MAX];
                int fPi0ID1[PID_2GAMMA_MAX];
                int fPi0ID2[PID_2GAMMA_MAX];
                int fPi0Tag1[PID_2GAMMA_MAX];
                int fPi0Tag2[PID_2GAMMA_MAX];
                float fPi0g1Px[PID_2GAMMA_MAX];
                float fPi0g1Py[PID_2GAMMA_MAX];
                float fPi0g1Pz[PID_2GAMMA_MAX];
                float fPi0g1E[PID_2GAMMA_MAX];
                float fPi0g2Px[PID_2GAMMA_MAX];
                float fPi0g2Py[PID_2GAMMA_MAX];
                float fPi0g2Pz[PID_2GAMMA_MAX];
                float fPi0g2E[PID_2GAMMA_MAX];

// PID two gamma fit: ate
                int fEta;
                float fEtaMtot;
                int fEtaID[PID_2GAMMA_MAX];
                float fEtaM[PID_2GAMMA_MAX];
                float fEtaE[PID_2GAMMA_MAX];
                float fEtaP[PID_2GAMMA_MAX];
                float fEtaThe[PID_2GAMMA_MAX];
                float fEtaPhi[PID_2GAMMA_MAX];
                float fEtaProb[PID_2GAMMA_MAX];
                float fEtaChi2[PID_2GAMMA_MAX];
                float fEtaNdf[PID_2GAMMA_MAX];
                float fEtaPul0[PID_2GAMMA_MAX];
                float fEtaPul1[PID_2GAMMA_MAX];
                float fEtaPul2[PID_2GAMMA_MAX];
                float fEtaPul3[PID_2GAMMA_MAX];
                float fEtaPul4[PID_2GAMMA_MAX];
                float fEtaPul5[PID_2GAMMA_MAX];
                int fEtaID1[PID_2GAMMA_MAX];
                int fEtaID2[PID_2GAMMA_MAX];
                int fEtaTag1[PID_2GAMMA_MAX];
                int fEtaTag2[PID_2GAMMA_MAX];
                float fEtag1Px[PID_2GAMMA_MAX];
                float fEtag1Py[PID_2GAMMA_MAX];
                float fEtag1Pz[PID_2GAMMA_MAX];
                float fEtag1E[PID_2GAMMA_MAX];
                float fEtag2Px[PID_2GAMMA_MAX];
                float fEtag2Py[PID_2GAMMA_MAX];
                float fEtag2Pz[PID_2GAMMA_MAX];
                float fEtag2E[PID_2GAMMA_MAX];



		 
		float FitProb[PID_2GAMMA_MAX];
		float FitChi2[PID_2GAMMA_MAX];
		float FitNDF[PID_2GAMMA_MAX];
	        float FitPull[25][PID_2GAMMA_MAX];

		float FitPull0[PID_2GAMMA_MAX];
		float FitPull1[PID_2GAMMA_MAX];
		float FitPull2[PID_2GAMMA_MAX];
		float FitPull3[PID_2GAMMA_MAX];
		float FitPull4[PID_2GAMMA_MAX];

		DTwoGammaFit* fit2g_PI0;
		DTwoGammaFit* fit2g_ETA;

		float eta_mass[PID_2GAMMA_MAX];
		float eta_prob[PID_2GAMMA_MAX];
		float eta_Chi2[PID_2GAMMA_MAX];
		float eta_pull_1[PID_2GAMMA_MAX];
		float eta_pull_2[PID_2GAMMA_MAX];
		float eta_pull_3[PID_2GAMMA_MAX];
		float eta_pull_4[PID_2GAMMA_MAX];
		float eta_pull_5[PID_2GAMMA_MAX];
		float eta_pull_6[PID_2GAMMA_MAX];
		float eta_momentum_Mag[PID_2GAMMA_MAX];
		float eta_energy[PID_2GAMMA_MAX];

		int nPairs;
                int nPhotons;
                int nPhotons_ps;

		float pi0_mass[PID_2GAMMA_MAX];
		float pi0_prob[PID_2GAMMA_MAX];
		float pi0_Chi2[PID_2GAMMA_MAX];
		float pi0_pull_1[PID_2GAMMA_MAX];
		float pi0_pull_2[PID_2GAMMA_MAX];
		float pi0_pull_3[PID_2GAMMA_MAX];
		float pi0_pull_4[PID_2GAMMA_MAX];
		float pi0_pull_5[PID_2GAMMA_MAX];
		float pi0_pull_6[PID_2GAMMA_MAX];
		float pi0_momentum_Mag[PID_2GAMMA_MAX];
		float pi0_energy[PID_2GAMMA_MAX];

		float gamma4_energy[PID_2GAMMA_MAX];
		float gamma4_pt[PID_2GAMMA_MAX];
		float total_pt[PID_2GAMMA_MAX];


		
		float ProbMax;
		float ProbMax_PI0;
	        float ProbMax_ETA;
	        float ProbMax_FIT;

		float PI0MASS;
		float ETAMASS;
		float PI0CHI2;
		float ETACHI2;
		float FITCHI2;
		float FITNDF;
		double PULL[25];

		float PULL0;
		float PULL1;
		float PULL2;
		float PULL3;
		float PULL4;


  
		float RECMASS;
		float RESMASS;
		float MISMASS;
		float MISMASS2;
 
		float RECMOM;
		float RESMOM;
		float MISMOM;
		
		float RESENERGY;
		float RECENERGY;
		float MISENERGY;
		
		float RECPT;
		float RESPT;
		float MISPT;
		float TOTALPT;
		
		float dphoton_energy[10];
		float dphoton_theta[10];

		float resphot_energy[4];
		float resphot_theta[4];
		float lowresphot_energy;
		float lowresphot_theta;

		float resphot_in_energy[4];
		float resphot_in_theta[4];
		float lowresphot_in_energy;
		float lowresphot_in_theta;


		float etapair_energydiff;
		float pi0pair_energydiff;


		float t;
		float cosGJ;
		float phiGJ;

		float pi0theta_g1;
		float pi0theta_g2;
		float pi0phi_g1;
		float pi0phi_g2;
		float pi0e_g1;
		float pi0e_g2;


		float pi0theta_g1_com[PID_2GAMMA_MAX];
		float pi0theta_g2_com[PID_2GAMMA_MAX];
		float pi0phi_g1_com[PID_2GAMMA_MAX];
		float pi0phi_g2_com[PID_2GAMMA_MAX];
		float pi0e_g1_com[PID_2GAMMA_MAX];
		float pi0e_g2_com[PID_2GAMMA_MAX];


		float etatheta_g1;
		float etatheta_g2;
		float etaphi_g1;
		float etaphi_g2;
		float etae_g1;
		float etae_g2;


		float etatheta_g1_com[PID_2GAMMA_MAX];
		float etatheta_g2_com[PID_2GAMMA_MAX];
		float etaphi_g1_com[PID_2GAMMA_MAX];
		float etaphi_g2_com[PID_2GAMMA_MAX];
		float etae_g1_com[PID_2GAMMA_MAX];
		float etae_g2_com[PID_2GAMMA_MAX];
      
        int m_eventCounter;
	int nCandChi2_3;
	int nCandChi2_5;
	int nFitCandChi2_50;
	int nFitCandProb;

        int nPart;
        float m_e[3];
        float m_px[3];
        float m_py[3];
        float m_pz[3];

        float m_eBeam;
        float m_pxBeam;
        float m_pyBeam;
        float m_pzBeam;

        float m_eRecoil;
        float m_pxRecoil;
        float m_pyRecoil;
        float m_pzRecoil;

	float neutEtot;

	TLorentzVector FITP4_RES;
	//       vector<DKinematicData> Tkd_initialState;
	//	//	vector<DKinematicData> Tkd_finalState;
	//	vector<DKinematicData> Tkindata_PI0;
	//	vector<DKinematicData> Tkinout_PI0;
	//	DTwoGammaFit* Tfit2g_PI0
	TMatrixD CovMat;
	TMatrixD CovMat_FIT;


};

#endif // _FCALTreeProc_

